package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.constant.CommonAttributes;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.common.utils.StringUtil;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.*;
import cn.wisenergy.model.enums.GradeType;
import cn.wisenergy.model.enums.SchemeTypeEnums;
import cn.wisenergy.model.enums.StudentClassEnum;
import cn.wisenergy.model.enums.StudentType;
import cn.wisenergy.model.vo.*;
import cn.wisenergy.service.app.SchemeService;
import cn.wisenergy.service.common.VolunteerManager;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author 86187
 * @ Description: 方案接口实现
 * @ Author     : 86187
 * @ Date       : 2021/1/13 15:04
 */
@Service
@Slf4j
public class SchemeServiceImpl extends ServiceImpl<SchemeMapper, SchemeInfo> implements SchemeService {
    @Autowired
    private UsersMapper usersMapper;

    @Autowired
    private SchemeMapper schemeMapper;

    @Autowired
    private ScoreInfoMapper scoreInfoMapper;

    @Autowired
    private VolunteerMapper volunteerMapper;

    @Autowired
    private UserVolunteerMapper userVolunteerMapper;

    @Autowired
    private AdmissionRuleMapper admissionRuleMapper;

    @Autowired
    private ProfessionMapper professionMapper;

    @Autowired
    private VolunteerManager volunteerManager;

    @Autowired
    private UserLimitMapper userLimitMapper;
    /**
     * 错误码
     */
    private static final int ERROR_CODE = 500;

    @Override
    public R<VolunteerVo> getList(SchemeVo queryVo) {
        log.info("volunteer-service[]SchemeServiceImpl[]getList[]input.param.queryVo:" + queryVo);
        if (null == queryVo || null == queryVo.getUserId() || null == queryVo.getCultureGrade()) {
            return R.error("入参为空!");
        }

        if (StringUtils.isBlank(queryVo.getProfessionIds())) {
            return R.error("未选择专业!");
        }

        //1、根据userId获取用户信息
        User user = usersMapper.getById(queryVo.getUserId());
        if (null == user) {
            return R.error("用户信息不存在!");
        }

        //2、获取考生成绩
        QueryWrapper<ScoreInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", queryVo.getUserId());
        ScoreInfo scoreInfo = scoreInfoMapper.selectOne(queryWrapper);
        if (null == scoreInfo) {
            return R.error("无考生成绩!");
        }

        //3、根据筛选方案算法，筛选方案
        //副科名称
        List<String> className = getClassName(scoreInfo);
        if (CollectionUtils.isEmpty(className)) {
            return R.error("考生无副科成绩!");
        }

        //选取专业名称
        List<String> professionName = getProfessionName(queryVo);
        if (CollectionUtils.isEmpty(professionName)) {
            return R.error("未选择专业!");
        }
        //本科or专科
        String gradeType = queryVo.getGradeType() == null ? null : queryVo.getGradeType();

        //(1)判断考生类型 文化课考生
        //初始化返回数据总条数
        int totalResult = 0;
        List<Volunteer> list = new ArrayList<>();
        VolunteerVo volunteerVo = new VolunteerVo();
        if (StudentType.CULTURE_STUDENT.getCode().equals(user.getExamType())) {
            //获取文化课本科一批规则
            AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_CULTURE.getCode());
            if (null == firstRule) {
                return R.error("无文化课本科录取规则!");
            }

            //获取文化课专科一批规则
            AdmissionRule secondRule = admissionRuleMapper.getByType(SchemeTypeEnums.JUNIOR_COLLEGE_MAJOR.getCode());
            if (null == secondRule) {
                return R.error("无文化课专科录取规则!");
            }

            //1).判断文化课的分数是否大于等于最大文化分
            //本科类比文化分 = 考生分数-( 本年录取分 -去年录取分)
            double firstCulture = Double.parseDouble(queryVo.getCultureGrade())-
                    (firstRule.getCurrentYearCulture() - firstRule.getCultureMin());
            double cultureMax = Double.parseDouble(firstRule.getCultureMax().toString());
            if (firstCulture >= cultureMax) {
                volunteerVo.setUserId(user.getId());
                volunteerVo.setVolunteers(list);
                return R.ok("恭喜您，因您的成绩突出，请联系官方免费领取VIP一对一报考方案指导！", volunteerVo);
            }

            //2).判断文化课的分数是否小于最小报考分数
            //专科类比文化分 = 本年录取分 -去年录取分 + 考生分数
            double secondCulture = Double.parseDouble(queryVo.getCultureGrade())-(secondRule.getCurrentYearCulture() - secondRule.getCultureMin()) ;
            double cultureMin = Double.parseDouble(secondRule.getCultureMin().toString());
            if (secondCulture < cultureMin) {
                return R.error("很抱歉，您的成绩不满足报考条件，无法查询!");
            }

            totalResult = firstRule.getNumber();
            list = getCultureList(firstRule, secondRule, className, professionName, firstCulture, secondCulture, gradeType);
        }

        //(2)美术考生
        if (StudentType.ART_STUDENT.getCode().equals(user.getExamType())) {
            //获取美术本科一批录取规则
            AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_ARTS.getCode());
            if (null == firstRule) {
                return R.error("无美术本科录取规则!");
            }

            //获取美术专科一批录取规则
            AdmissionRule secondRule = admissionRuleMapper.getByType(SchemeTypeEnums.JUNIOR_COLLEGE_ARTS.getCode());
            if (null == secondRule) {
                return R.error("无美术专科录取规则!");
            }

            //判断考生是否有报取资格
            //专科类比分 = 考生分数-( 本年录取分 -去年录取分)
            double secondCulture = Double.parseDouble(queryVo.getCultureGrade())-
                    (secondRule.getCurrentYearCulture() - secondRule.getCultureMin());
            double secondMajor =Double.parseDouble(queryVo.getMajorGrade())-
                    (secondRule.getCurrentYearMajor() - secondRule.getProfessionMin());
            if (secondCulture < secondRule.getCultureMin() || secondMajor < secondRule.getProfessionMin()) {
                return R.error("很抱歉，您的成绩不满足报考条件，无法查询!");
            }

            totalResult = firstRule.getNumber();
            list = getArtsList(queryVo, firstRule, secondRule, className, professionName, gradeType);
        }

        //(3)文学编导考生
        if (StudentType.LITERATURE_STUDENT.getCode().equals(user.getExamType())) {
            //获取文学编导本科一批录取规则
            AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_LITERATURE.getCode());
            if (null == firstRule) {
                return R.error("无文学编导本科录取规则!");
            }

            //获取文学编导专科一批录取规则
            AdmissionRule secondRule = admissionRuleMapper.getByType(SchemeTypeEnums.JUNIOR_COLLEGE_LITERATURE.getCode());
            if (null == secondRule) {
                return R.error("无文学编导专科录取规则!");
            }

            //判断考生是否有报取资格
            //专科类比分 =考生分数-( 本年录取分 -去年录取分)
            double secondCulture = Double.parseDouble(queryVo.getCultureGrade()) -
                    (secondRule.getCurrentYearCulture() - secondRule.getCultureMin() );
            double secondMajor =Double.parseDouble(queryVo.getMajorGrade())-
                    (secondRule.getCurrentYearMajor() - secondRule.getProfessionMin() );
            if (secondCulture < secondRule.getCultureMin() || secondMajor < secondRule.getProfessionMin()) {
                return R.error("很抱歉，您的专业成绩不满足报考条件，无法查询!");
            }

            totalResult = firstRule.getNumber();
            list = getLiteratureList(queryVo, firstRule, secondRule, className, professionName, gradeType);
        }

        //(4)体育考生
        if (StudentType.SPORTS_STUDENT.getCode().equals(user.getExamType())) {
            //获取体育本科一批录取规则
            AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_SPORTS.getCode());
            if (null == firstRule) {
                return R.error("无体育本科录取规则!");
            }
            //获取体育专科一批录取规则
            AdmissionRule secondRule = admissionRuleMapper.getByType(SchemeTypeEnums.JUNIOR_COLLEGE_SPORTS.getCode());
            if (null == secondRule) {
                return R.error("无体育专科录取规则!");
            }

            //判断考生是否有报取资格
            //综合成绩差值
            double value = secondRule.getCurrentYearCulture() - secondRule.getCultureMin();
            double culture = Double.parseDouble(queryVo.getCultureGrade());
            double major = Double.parseDouble(queryVo.getMajorGrade());

            //计算综合成绩
            double total = major * 750 / 100 * 0.7 + culture * 0.3 - value;
            if (total < secondRule.getCultureMin()) {
                return R.error("很抱歉，您的成绩不满足报考条件，无法查询!");
            }

            totalResult = firstRule.getNumber();
            list = getSportsList(culture,major, firstRule, secondRule, className, professionName, gradeType);
        }

        //4、判断返回的结果数据是否大于totalResult
        if (list.size() > totalResult) {
            list = getVolunteerList(list, totalResult);
        }

        //排序
        list.sort(Comparator.comparing(Volunteer::getLowestMark).reversed());

        //5判断是否展示全部方案
        if (queryVo.getIsShowAll() == 0) {
            List<Volunteer> result;
            if (list.size() < 10) {
                result = list.subList(0, list.size());
            } else {
                result = list.subList(0, 10);
            }
            volunteerVo.setUserId(user.getId());
            //设置查询时间
            SetQueryTime(result);
            volunteerVo.setVolunteers(result);
            List<Volunteer> volunteers = volunteerVo.getVolunteers();
            //为空的最低位次用--代替
            for (Volunteer info : volunteers) {
                if (StringUtils.isBlank(info.getLowestRank())) {
                    info.setLowestRank("--");
                }
            }
            return R.ok(volunteerVo);
        }

        //获取用户查询次数信息
        UserLimit userLimit = userLimitMapper.getByUserId(user.getId());
        if (null == userLimit) {
            return R.error("用户查询信息不存在!");
        }

        if (userLimit.getUsableLimit() == 0) {
            return R.error(1, "您的查询次数已使用完了，请充值!");
        }

        //6保存用户方案记录、志愿之间的关联关系,扣减查询次数
        R<Integer> bool = volunteerManager.saveUserVolunteer(user, list, scoreInfo, queryVo.getBigMajorNames(), userLimit);
        if (null != bool && bool.getCode() == ERROR_CODE) {
            return R.error(bool.getMessage());
        }

        volunteerVo.setUserId(user.getId());
        assert bool != null;
        volunteerVo.setRecordId(bool.getData());

        //设置查询时间
        SetQueryTime(list);
        volunteerVo.setVolunteers(list);
        volunteerVo.setTotal(list.size());
        //为空的最低位次用--代替
        List<Volunteer> volunteers = volunteerVo.getVolunteers();
        for (Volunteer info : volunteers) {
            if (StringUtils.isBlank(info.getLowestRank())) {
                info.setLowestRank("--");
            }
        }
        return R.ok(volunteerVo);
    }

    @Override
    public R<Boolean> deleteById(Integer schemeId) {
        log.info("volunteer-service[]SchemeServiceImpl[]deleteById[]input.param.schemeId:" + schemeId);
        if (null == schemeId) {
            return R.error("入参为空!");
        }
        //根据方案id,获取方案信息
        QueryWrapper<SchemeInfo> query = new QueryWrapper<>();
        query.eq("id", schemeId);
        query.eq("is_delete", 0);
        SchemeInfo schemeInfo = baseMapper.selectOne(query);
        if (null == schemeInfo) {
            return R.error("方案信息不存在!");
        }
        //更新方案状态为 ：1 删除
        schemeInfo.setIsDelete(1);

        //根据方案id,获取志愿信息
        QueryWrapper<Volunteer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("scheme_id", schemeId);
        queryWrapper.eq("is_delete", 0);
        List<Volunteer> list = volunteerMapper.selectList(queryWrapper);

        boolean bool = volunteerManager.deleteSchemeAndVolunteer(schemeInfo, schemeId, list.size());
        return R.ok(bool);
    }

    @Override
    public R<PageInfo<SchemeInfo>> getSchemeList(SchemeQueryVo schemeVo) {
        log.info("volunteer-service[]SchemeServiceImpl[]getSchemeList[]input.param.schemeVo:" + schemeVo);
        if (null == schemeVo) {
            return R.error("入参为空!");
        }
        pageHandle(schemeVo);
        int total = schemeMapper.count();
        Map<String, Object> map = new HashMap<>(16);

        map.put("startNum", schemeVo.getStartNum());
        map.put("endNum", schemeVo.getEndNum());
        List<SchemeInfo> list = schemeMapper.getList(map);
        PageInfo<SchemeInfo> info = new PageInfo<>();
        info.setPageSize(schemeVo.getPageSize());
        info.setPageNum(schemeVo.getPageNo());
        info.setTotal(total);
        info.setList(list);
        return R.ok(info);
    }

    @Override
    public R<String> download(Integer schemeId) {
        log.info("volunteer-service[]SchemeServiceImpl[]download[]input.param.schemeId:" + schemeId);
        if (null == schemeId) {
            return R.error("入参为空!");
        }

        //获取方案信息
        SchemeInfo schemeInfo = schemeMapper.getById(schemeId);
        if (null == schemeInfo || StringUtils.isBlank(schemeInfo.getFileUrl())) {
            return R.error("未上传方案，无法下载!");
        }

        return R.ok(schemeInfo.getFileUrl());
    }

    @Override
    public List<Integer> ceishi(Integer id) {

        List<UserVolunteer> list = userVolunteerMapper.listById(id);
        List<Integer> ids = list.stream().map(UserVolunteer::getVolunteerId).collect(Collectors.toList());
        List<Integer> integerList = new ArrayList<>();
        List<Integer> result = new ArrayList<>();
        for (Integer vid : ids) {
            if (integerList.contains(vid)) {
                result.add(vid);
            } else {
                integerList.add(vid);
            }
        }
        return result;
    }

    @Override
    public R<VolunteerVo> getListVip(SchemeVipVo schemeVo) {
        log.info("volunteer-service[]getListVip[]getList[]input.param:" + schemeVo);
        if (null == schemeVo || null == schemeVo.getUserId()) {
            return R.error("入参为空!");
        }

        //综合成绩区间
        double totalMax = schemeVo.getTotalMax();
        double totalMin = schemeVo.getTotalMin();

        //地区
        List<String> locals = StringUtil.strToArrayStr(schemeVo.getLocal());

        //1、根据userId获取用户信息
        User user = usersMapper.getById(schemeVo.getUserId());
        if (null == user) {
            return R.error("请先添加vip客户信息!");
        }

        //2、获取考生成绩
        QueryWrapper<ScoreInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", schemeVo.getUserId());
        ScoreInfo scoreInfo = scoreInfoMapper.selectOne(queryWrapper);
        if (null == scoreInfo) {
            return R.error("无vip客户成绩!");
        }

        //3、副科名称
        List<String> className = getClassName(scoreInfo);
        if (CollectionUtils.isEmpty(className)) {
            return R.error("考生无副科成绩!");
        }
        //本科or专科
        String gradeType = StringUtils.isBlank(schemeVo.getGradeType()) ? null : schemeVo.getGradeType();
        //文化分
        Double majorGrade = scoreInfo.getMajorGrade();
        //专业分
        Double cultureGrade = scoreInfo.getCultureGrade();


        //选取专业名称;
        List<Integer> professionIds = StringUtil.strToArray(schemeVo.getProfessionIds());
        List<String> nameByIds = professionMapper.getNameByIds(professionIds);
        if (CollectionUtils.isEmpty(nameByIds)) {
            return R.error("未选择专业!");
        }

        //(1)判断考生类型 文化课考生
        List<Volunteer> list = new ArrayList<>();
        VolunteerVo volunteerVo = new VolunteerVo();
        if (StudentType.CULTURE_STUDENT.getCode().equals(user.getExamType())) {
            //获取文化课本科一批规则
            AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_CULTURE.getCode());
            if (null == firstRule) {
                return R.error("无文化课本科录取规则!");
            }

            //获取文化课专科一批规则
            AdmissionRule secondRule = admissionRuleMapper.getByType(SchemeTypeEnums.JUNIOR_COLLEGE_MAJOR.getCode());
            if (null == secondRule) {
                return R.error("无文化课专科录取规则!");
            }

            if (cultureGrade < secondRule.getCultureMin()) {
                return R.error("很抱歉，客户的成绩不满足报考条件，无法查询!");
            }
            list = getCultureVipList(firstRule, cultureGrade, totalMax, totalMin, locals, nameByIds, className, gradeType);
        }

        //(2)美术考生
        if (StudentType.ART_STUDENT.getCode().equals(user.getExamType())) {
            //获取美术本科一批录取规则
            AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_ARTS.getCode());
            if (null == firstRule) {
                return R.error("无美术本科录取规则!");
            }

            //获取美术专科一批录取规则
            AdmissionRule secondRule = admissionRuleMapper.getByType(SchemeTypeEnums.JUNIOR_COLLEGE_ARTS.getCode());
            if (null == secondRule) {
                return R.error("无美术专科录取规则!");
            }

            if (majorGrade < secondRule.getProfessionMin()) {
                return R.error("很抱歉，客户的成绩不满足报考条件，无法查询!");
            }
            list = getArtsVipList(firstRule, cultureGrade, majorGrade, totalMax, totalMin, locals, nameByIds, className, gradeType);
        }

        //(3)文学编导考生
        if (StudentType.LITERATURE_STUDENT.getCode().equals(user.getExamType())) {
            //获取文学编导本科一批录取规则
            AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_LITERATURE.getCode());
            if (null == firstRule) {
                return R.error("无文学编导本科录取规则!");
            }

            //获取文学编导专科一批录取规则
            AdmissionRule secondRule = admissionRuleMapper.getByType(SchemeTypeEnums.JUNIOR_COLLEGE_LITERATURE.getCode());
            if (null == secondRule) {
                return R.error("无文学编导专科录取规则!");
            }

            if (majorGrade < secondRule.getProfessionMin()) {
                return R.error("很抱歉，客户的成绩不满足报考条件，无法查询!");
            }
            list = getLiteratureVipList(firstRule, cultureGrade, majorGrade, totalMax, totalMin, locals, nameByIds, className, gradeType);
        }

        //(4)体育考生
        if (StudentType.SPORTS_STUDENT.getCode().equals(user.getExamType())) {
            //获取体育本科一批录取规则
            AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_SPORTS.getCode());
            if (null == firstRule) {
                return R.error("无体育本科录取规则!");
            }

            //获取体育专科一批录取规则
            AdmissionRule secondRule = admissionRuleMapper.getByType(SchemeTypeEnums.JUNIOR_COLLEGE_SPORTS.getCode());
            if (null == secondRule) {
                return R.error("无体育专科录取规则!");
            }
            double total = majorGrade * 750 / 100 * 0.7 + cultureGrade * 0.3;
            if (total < secondRule.getCultureMin()) {
                return R.error("很抱歉，客户的成绩不满足报考条件，无法查询!");
            }

            list = getSportsVipList(firstRule, cultureGrade, majorGrade, totalMax, totalMin, locals, nameByIds, className, gradeType);
        }

        //4判断是否展示全部方案
        if (schemeVo.getIsShowAll() == 0) {
            List<Volunteer> result;
            if (list.size() < 10) {
                result = list.subList(0, list.size());
            } else {
                result = list.subList(0, 10);
            }
            //筛选出带和的字符串的限专业学校
            Iterator<Volunteer> iterator = list.iterator();
            while (iterator.hasNext()) {
                Volunteer info = iterator.next();
                if (info.getCourseDemand().contains("和")) {
                    String[] course = info.getCourseDemand().split("和");
                    String classStr = className.stream().map(String::valueOf).collect(Collectors.joining(","));
                    for (String co : course) {
                        if (!classStr.contains(co)) {
                            iterator.remove();
                            break;
                        }
                    }
                }
            }
            //设置查询时间
            SetQueryTime(result);
            volunteerVo.setUserId(user.getId());
            volunteerVo.setVolunteers(result);
            return R.ok(volunteerVo);
        }

        //排序
        list.sort(Comparator.comparing(Volunteer::getLowestMark).reversed());
        volunteerVo.setUserId(user.getId());
        //筛选出带和的字符串的限专业学校
        Iterator<Volunteer> iterator = list.iterator();
        while (iterator.hasNext()) {
            Volunteer info = iterator.next();
            if (info.getCourseDemand().contains("和")) {
                String[] course = info.getCourseDemand().split("和");
                String classStr = className.stream().map(String::valueOf).collect(Collectors.joining(","));
                for (String co : course) {
                    if (!classStr.contains(co)) {
                        iterator.remove();
                        break;
                    }
                }
            }
        }

        //设置查询时间
        SetQueryTime(list);
        volunteerVo.setVolunteers(list);
        volunteerVo.setTotal(list.size());
        return R.ok(volunteerVo);
    }

    /**
     * 获取vip文化生志愿列表
     *
     * @param firstRule       录取规则
     * @param professionNames 专业名称
     * @return vip文化生志愿列表
     */
    private List<Volunteer> getCultureVipList(AdmissionRule firstRule, double cultureGrade, double totalMax, double totalMin,
                                              List<String> locals, List<String> professionNames, List<String> classNames, String gradeType) {
        double cultureMax = firstRule.getCultureMax();
        double cultureMin = firstRule.getCultureMin();

        //考生的分数在本科一批中
        List<Volunteer> list;
        Map<String, Object> map = new HashMap<>(16);
        map.put("professionNames", professionNames);
        map.put("locals", locals);
        map.put("classNames", classNames);
        boolean type = !StringUtils.isBlank(gradeType) && GradeType.UNDERGRADUATE.getDescription().equals(gradeType);
        boolean grade = StringUtils.isBlank(gradeType) && cultureGrade <= cultureMax && cultureGrade >= cultureMin;
        if (type || grade) {
            map.put("type", SchemeTypeEnums.UNDERGRADUATE_CULTURE.getCode());
            map.put("totalMax", totalMax);
            map.put("totalMin", totalMin);
            list = volunteerMapper.getVolunteerListVip(map);
        } else {
            //获取专科方案
            map.put("type", SchemeTypeEnums.JUNIOR_COLLEGE_MAJOR.getCode());
            map.put("totalMax", totalMax);
            map.put("totalMin", totalMin);
            list = volunteerMapper.getVolunteerListVip(map);
        }
        return list;
    }

    /**
     * 获取vip美术生志愿方案
     *
     * @param firstRule       本科规则
     * @param professionNames 专业名称
     * @return 志愿方案列表
     */
    private List<Volunteer> getArtsVipList(AdmissionRule firstRule, double cultureGrade, double majorGrade, double totalMax, double totalMin, List<String> locals,
                                           List<String> professionNames, List<String> classNames, String gradeType) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("professionNames", professionNames);
        map.put("locals", locals);
        map.put("classNames", classNames);
        //1、考生是否能报取美术本科
        List<Volunteer> list;
        boolean type = !StringUtils.isBlank(gradeType) && GradeType.UNDERGRADUATE.getDescription().equals(gradeType);
        boolean grade = StringUtils.isBlank(gradeType) && cultureGrade >= firstRule.getCultureMin() && majorGrade >= firstRule.getProfessionMin();
        if (type || grade) {
            map.put("type", SchemeTypeEnums.UNDERGRADUATE_ARTS.getCode());
            map.put("totalMax", totalMax);
            map.put("totalMin", totalMin);
            list = volunteerMapper.getVolunteerListVip(map);
        } else {
            //获取专科方案
            map.put("type", SchemeTypeEnums.JUNIOR_COLLEGE_ARTS.getCode());
            map.put("totalMax", totalMax);
            map.put("totalMin", totalMin);
            list = volunteerMapper.getVolunteerListVip(map);
        }
        return list;
    }

    /**
     * 获取vip文学编导生志愿方案
     *
     * @param firstRule       本科规则
     * @param professionNames 专业名称
     * @return 志愿方案列表
     */
    private List<Volunteer> getLiteratureVipList(AdmissionRule firstRule, double cultureGrade, double majorGrade, double totalMax, double totalMin, List<String> locals,
                                                 List<String> professionNames, List<String> classNames, String gradeType) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("professionNames", professionNames);
        map.put("locals", locals);
        map.put("classNames", classNames);
        //1、考生是否能报取美术本科
        List<Volunteer> list;
        boolean type = !StringUtils.isBlank(gradeType) && GradeType.UNDERGRADUATE.getDescription().equals(gradeType);
        boolean grade = StringUtils.isBlank(gradeType) && cultureGrade >= firstRule.getCultureMin() && majorGrade >= firstRule.getProfessionMin();
        if (type || grade) {
            map.put("type", SchemeTypeEnums.UNDERGRADUATE_LITERATURE.getCode());
            map.put("totalMax", totalMax);
            map.put("totalMin", totalMin);
            map.put("locals", locals);
            list = volunteerMapper.getVolunteerListVip(map);
        } else {
            //获取专科方案
            map.put("type", SchemeTypeEnums.JUNIOR_COLLEGE_LITERATURE.getCode());
            map.put("totalMax", totalMax);
            map.put("totalMin", totalMin);
            map.put("locals", locals);
            list = volunteerMapper.getVolunteerListVip(map);
        }
        return list;
    }

    /**
     * 获取vip体育生生志愿方案
     *
     * @param totalMax        考生综合成绩最高分
     * @param firstRule       本科规则
     * @param professionNames 专业名称
     * @return 志愿方案列表
     */
    private List<Volunteer> getSportsVipList(AdmissionRule firstRule, double cultureGrade, double majorGrade, double totalMax, double totalMin, List<String> locals,
                                             List<String> professionNames, List<String> classNames, String gradeType) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("professionNames", professionNames);
        map.put("locals", locals);
        map.put("classNames", classNames);
        //1、考生是否能报取体育本科
        List<Volunteer> list;
        boolean type = !StringUtils.isBlank(gradeType) && GradeType.UNDERGRADUATE.getDescription().equals(gradeType);
        boolean grade = StringUtils.isBlank(gradeType) && cultureGrade >= firstRule.getCultureMin() && majorGrade > firstRule.getProfessionMin();
        if (type || grade) {
            map.put("type", SchemeTypeEnums.UNDERGRADUATE_SPORTS.getCode());
            map.put("totalMax", totalMax);
            map.put("totalMin", totalMin);
            map.put("locals", locals);
            list = volunteerMapper.getVolunteerListVip(map);
        } else {
            //获取专科方案
            map.put("type", SchemeTypeEnums.JUNIOR_COLLEGE_SPORTS.getCode());
            map.put("totalMax", totalMax);
            map.put("totalMin", totalMin);
            map.put("locals", locals);
            list = volunteerMapper.getVolunteerListVip(map);
        }


        return list;
    }

    /**
     * 获取文化生志愿列表
     *
     * @param firstRule       录取规则
     * @param classNames      副科名称
     * @param professionNames 专业名称
     * @return 文化生志愿列表
     */
    private List<Volunteer> getCultureList(AdmissionRule firstRule, AdmissionRule secondRule,
                                           List<String> classNames, List<String> professionNames,
                                           double firstCulture, double secondCulture, String gradeType) {
        double cultureMax = firstRule.getCultureMax();
        double cultureMin = firstRule.getCultureMin();

        //考生的分数在本科一批中
        List<Volunteer> list;
        Map<String, Object> map = new HashMap<>(16);
        map.put("classNames", classNames);
        map.put("professionNames", professionNames);
        boolean type = !StringUtils.isBlank(gradeType) && GradeType.UNDERGRADUATE.getDescription().equals(gradeType);
        boolean grade = StringUtils.isBlank(gradeType) && firstCulture < cultureMax && firstCulture >= cultureMin;
        if (type || grade) {
            double upGrade = firstCulture + firstRule.getUpMark();
            double downGrade = firstCulture - firstRule.getDownMark();
            map.put("upGrade", upGrade);
            map.put("type", SchemeTypeEnums.UNDERGRADUATE_CULTURE.getCode());
            map.put("downGrade", downGrade);
            list = volunteerMapper.getVolunteerList(map);
            String classStr = classNames.stream().map(String::valueOf).collect(Collectors.joining(","));
            //筛选出带和的字符串的限专业学校并进行判断
            Iterator<Volunteer> iterator = list.iterator();
            while (iterator.hasNext()) {
                Volunteer info = iterator.next();
                if (info.getCourseDemand().contains("和")) {
                    String[] course = info.getCourseDemand().split("和");
                    for (String co : course) {
                        if (!classStr.contains(co)) {
                            iterator.remove();
                            break;
                        }
                    }
                }
            }
            if (list.size() < firstRule.getNumber()) {
                List<Volunteer> fillList = fillUpVolunteer(list, firstRule.getNumber(), upGrade, downGrade,
                        SchemeTypeEnums.UNDERGRADUATE_CULTURE.getCode(), classNames);
                list.addAll(fillList);
            }
        } else {
            //考生的分数是在专科一批中
            double upGrade = secondCulture + secondRule.getUpMark();
            double downGrade = secondCulture - secondRule.getDownMark();
            map.put("upGrade", upGrade);
            map.put("type", SchemeTypeEnums.JUNIOR_COLLEGE_MAJOR.getCode());
            map.put("downGrade", downGrade);
            list = volunteerMapper.getVolunteerList(map);
            String classStr = classNames.stream().map(String::valueOf).collect(Collectors.joining(","));
            //筛选出带和的字符串的限专业学校并进行判断
            Iterator<Volunteer> iterator = list.iterator();
            while (iterator.hasNext()) {
                Volunteer info = iterator.next();
                if (info.getCourseDemand().contains("和")) {
                    String[] course = info.getCourseDemand().split("和");
                    for (String co : course) {
                        if (!classStr.contains(co)) {
                            iterator.remove();
                            break;
                        }
                    }
                }
            }
            //如果数据不满足最大数量，取不限专业最低分补充
            if (list.size() < secondRule.getNumber()) {
                List<Volunteer> fillList = fillUpVolunteer(list, secondRule.getNumber(), upGrade, downGrade,
                        SchemeTypeEnums.JUNIOR_COLLEGE_MAJOR.getCode(), classNames);
                list.addAll(fillList);
            }
        }
        return list;
    }

    /**
     * 获取美术生志愿方案
     *
     * @param queryVo         考生成绩
     * @param firstRule       本科规则
     * @param secondRule      专科规则
     * @param classNames      副科名称
     * @param professionNames 专业名称
     * @return 志愿方案列表
     */
    private List<Volunteer> getArtsList(SchemeVo queryVo, AdmissionRule firstRule, AdmissionRule secondRule,
                                        List<String> classNames, List<String> professionNames, String gradeType) {
        //本科类比分 = 考生分数-( 本年录取分 -去年录取分)
        double culture = Double.parseDouble(queryVo.getCultureGrade()) -
                (firstRule.getCurrentYearCulture() - firstRule.getCultureMin());
        double major = Double.parseDouble(queryVo.getMajorGrade()) -
                (firstRule.getCurrentYearMajor() - firstRule.getProfessionMin()) ;
        //计算综合成绩
        double total = major * 750 / 300 * 0.7 + culture * 0.3;
        BigDecimal   b   =   new   BigDecimal(total);
        total  =   b.setScale(2,   RoundingMode.HALF_UP).doubleValue();

        Map<String, Object> map = new HashMap<>(16);
        map.put("number", firstRule.getNumber());
        map.put("classNames", classNames);
        map.put("professionNames", professionNames);

        //1、考生是否能报取美术本科
        List<Volunteer> list = new ArrayList<>();
        boolean type = !StringUtils.isBlank(gradeType) && GradeType.UNDERGRADUATE.getDescription().equals(gradeType);
        boolean grade = StringUtils.isBlank(gradeType) && culture >= firstRule.getCultureMin() && major >= firstRule.getProfessionMin();
        if (type || grade) {
            //浮动分数
            double upGrade = total + firstRule.getUpMark();
            double downGrade = total - firstRule.getDownMark();
            map.put("upGrade", upGrade);
            map.put("downGrade", downGrade);
            map.put("type", SchemeTypeEnums.UNDERGRADUATE_ARTS.getCode());
            list = volunteerMapper.getVolunteerList(map);
            String classStr = classNames.stream().map(String::valueOf).collect(Collectors.joining(","));
            //筛选出带和的字符串的限专业学校并进行判断
            Iterator<Volunteer> iterator = list.iterator();
            while (iterator.hasNext()) {
                Volunteer info = iterator.next();
                if (info.getCourseDemand().contains("和")) {
                    String[] course = info.getCourseDemand().split("和");
                    for (String co : course) {
                        if (!classStr.contains(co)) {
                            iterator.remove();
                            break;
                        }
                    }
                }
            }
            //如果数据不满足最大数量，取不限专业最低分补充
            if (list.size() < secondRule.getNumber()) {
                List<Volunteer> fillList = fillUpVolunteer(list, firstRule.getNumber(), upGrade, downGrade,
                        SchemeTypeEnums.UNDERGRADUATE_ARTS.getCode(), classNames);
                list.addAll(fillList);
            }
            return list;
        }


        //专科科类比分 = 本年录取分 -去年录取分 + 考生分数
        double secondCulture = Double.parseDouble(queryVo.getCultureGrade()) -
                (secondRule.getCurrentYearCulture() - secondRule.getCultureMin() );
        double secondMajor = Double.parseDouble(queryVo.getMajorGrade()) -
                (secondRule.getCurrentYearMajor() - secondRule.getProfessionMin() );

        //计算综合成绩
        double secondRuleTotal = secondMajor * 750 / 300 * 0.7 + secondCulture * 0.3;
        BigDecimal   rel   =   new   BigDecimal(secondRuleTotal);
        secondRuleTotal  =   rel.setScale(2,   RoundingMode.HALF_UP).doubleValue();
        //获取该类型录取最高分数
        double high = volunteerMapper.getMaxLowestMark(SchemeTypeEnums.JUNIOR_COLLEGE_ARTS.getCode());

        //2、考生是否能报取美术专科
        if (secondCulture >= secondRule.getCultureMin() && secondMajor >= secondRule.getProfessionMin()) {
            //浮动分数
            double upGrade = secondRuleTotal + secondRule.getUpMark();
            double downGrade = secondRuleTotal - secondRule.getDownMark();
            if (secondRuleTotal < high) {
                map.put("upGrade", upGrade);
                map.put("downGrade", downGrade);
            }
            map.put("type", SchemeTypeEnums.JUNIOR_COLLEGE_ARTS.getCode());
            list = volunteerMapper.getVolunteerList(map);
            String classStr = classNames.stream().map(String::valueOf).collect(Collectors.joining(","));
            //筛选出带和的字符串的限专业学校并进行判断
            Iterator<Volunteer> iterator = list.iterator();
            while (iterator.hasNext()) {
                Volunteer info = iterator.next();
                if (info.getCourseDemand().contains("和")) {
                    String[] course = info.getCourseDemand().split("和");
                    for (String co : course) {
                        if (!classStr.contains(co)) {
                            iterator.remove();
                            break;
                        }
                    }
                }
            }
            //如果数据不满足最大数量，取不限专业最低分补充
            if (list.size() < secondRule.getNumber()) {
                List<Volunteer> fillList = fillUpVolunteer(list, secondRule.getNumber(), upGrade, downGrade,
                        SchemeTypeEnums.JUNIOR_COLLEGE_ARTS.getCode(), classNames);
                list.addAll(fillList);
            }
        }
        return list;
    }

    /**
     * 获取文学编导生志愿方案
     *
     * @param queryVo         考生成绩
     * @param firstRule       本科规则
     * @param secondRule      专科规则
     * @param classNames      副科名称
     * @param professionNames 专业名称
     * @return 志愿方案列表
     */
    private List<Volunteer> getLiteratureList(SchemeVo queryVo, AdmissionRule firstRule, AdmissionRule secondRule,
                                              List<String> classNames, List<String> professionNames, String gradeType) {
        //本科类比分 = 本年录取分 -去年录取分 + 考生分数
        double culture = Double.parseDouble(queryVo.getCultureGrade()) -
                (firstRule.getCurrentYearCulture() - firstRule.getCultureMin());
        double major = Double.parseDouble(queryVo.getMajorGrade()) -
                (firstRule.getCurrentYearMajor() - firstRule.getProfessionMin()) ;
        //计算综合成绩
        double total = major * 750 / 300 * 0.3 + culture * 0.7;
        BigDecimal   b   =   new   BigDecimal(total);
        total  =   b.setScale(2,   RoundingMode.HALF_UP).doubleValue();
        Map<String, Object> map = new HashMap<>(16);
        map.put("number", firstRule.getNumber());
        map.put("classNames", classNames);
        map.put("professionNames", professionNames);

        //1、考生是否能报取编导专科本科
        List<Volunteer> list;
        boolean type = !StringUtils.isBlank(gradeType) && GradeType.UNDERGRADUATE.getDescription().equals(gradeType);
        boolean grade = StringUtils.isBlank(gradeType) && culture >= firstRule.getCultureMin() && major >= firstRule.getProfessionMin();
        if (type || grade) {
            //浮动分数
            double upGrade = total + firstRule.getUpMark();
            double downGrade = total - firstRule.getDownMark();
            map.put("upGrade", upGrade);
            map.put("downGrade", downGrade);
            map.put("type", SchemeTypeEnums.UNDERGRADUATE_LITERATURE.getCode());
            list = volunteerMapper.getVolunteerList(map);
            String classStr = classNames.stream().map(String::valueOf).collect(Collectors.joining(","));
            //筛选出带和的字符串的限专业学校并进行判断
            Iterator<Volunteer> iterator = list.iterator();
            while (iterator.hasNext()) {
                Volunteer info = iterator.next();
                if (info.getCourseDemand().contains("和")) {
                    String[] course = info.getCourseDemand().split("和");
                    for (String co : course) {
                        if (!classStr.contains(co)) {
                            iterator.remove();
                            break;
                        }
                    }
                }
            }
            //如果数据不满足最大数量，取不限专业最低分补充
            if (list.size() < secondRule.getNumber()) {
                List<Volunteer> fillList = fillUpVolunteer(list, secondRule.getNumber(), upGrade, downGrade,
                        SchemeTypeEnums.UNDERGRADUATE_LITERATURE.getCode(), classNames);
                list.addAll(fillList);
            }
            return list;
        } else {
            //文学编导专科一批返回全部数据
            list = volunteerMapper.getVolunteerByType(SchemeTypeEnums.JUNIOR_COLLEGE_LITERATURE.getCode());
        }
        return list;
    }

    /**
     * 获取体育生生志愿方案
     *
     * @param culture           考生文化成绩
     * @param major           考生专业成绩
     * @param firstRule       本科规则
     * @param secondRule      专科规则
     * @param classNames      副科名称
     * @param professionNames 专业名称
     * @return 志愿方案列表
     */
    private List<Volunteer> getSportsList(double culture,double major, AdmissionRule firstRule, AdmissionRule secondRule,
                                          List<String> classNames, List<String> professionNames, String gradeType) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("number", firstRule.getNumber());
        map.put("classNames", classNames);
        map.put("professionNames", professionNames);
        double value = firstRule.getCurrentYearCulture() - firstRule.getCultureMin();
        double total= major * 750 / 100 * 0.7 + culture * 0.3 - value;
        BigDecimal   b   =   new   BigDecimal(total);
        total  =   b.setScale(2,   RoundingMode.HALF_UP).doubleValue();
        //1、考生是否能报取体育本科
        List<Volunteer> list = new ArrayList<>();
        boolean type = !StringUtils.isBlank(gradeType) && GradeType.UNDERGRADUATE.getDescription().equals(gradeType);
        boolean grade = StringUtils.isBlank(gradeType) && total >= firstRule.getCultureMin();
        if (type || grade) {
            //浮动分数
            double upGrade = total + firstRule.getUpMark();
            double downGrade = total - firstRule.getDownMark();
            if (downGrade < firstRule.getCultureMin()) {
                downGrade = firstRule.getCultureMin();
            }
            map.put("upGrade", upGrade);
            map.put("downGrade", downGrade);
            map.put("type", SchemeTypeEnums.UNDERGRADUATE_SPORTS.getCode());
            list = volunteerMapper.getVolunteerList(map);
            String classStr = classNames.stream().map(String::valueOf).collect(Collectors.joining(","));
            //筛选出带和的字符串的限专业学校并进行判断
            Iterator<Volunteer> iterator = list.iterator();
            while (iterator.hasNext()) {
                Volunteer info = iterator.next();
                if (info.getCourseDemand().contains("和")) {
                    String[] course = info.getCourseDemand().split("和");
                    for (String co : course) {
                        if (!classStr.contains(co)) {
                            iterator.remove();
                            break;
                        }
                    }
                }
            }
            //如果数据不满足最大数量，取不限专业最低分补充
            if (list.size() < secondRule.getNumber()) {
                List<Volunteer> fillList = fillUpVolunteer(list, secondRule.getNumber(), upGrade, downGrade,
                        SchemeTypeEnums.UNDERGRADUATE_SPORTS.getCode(), classNames);
                list.addAll(fillList);
            }
            return list;
        }

        //2、考生是否能报取体育专科
        if (total >= secondRule.getCultureMin()) {
            list = volunteerMapper.getVolunteerByType(SchemeTypeEnums.JUNIOR_COLLEGE_SPORTS.getCode());
        }

        return list;
    }

    /**
     * 获取副科的名称
     *
     * @param scoreInfo 成绩信息
     * @return 副科列表
     */
    private List<String> getClassName(ScoreInfo scoreInfo) {
        List<String> name = new ArrayList<>();
        name.add(StudentClassEnum.UNLIMITED.getDesc());
        if (null != scoreInfo.getPhysicsGrade() && scoreInfo.getPhysicsGrade() >= 0) {
            name.add(StudentClassEnum.PHYSICS_GRADE.getDesc());
        }

        if (null != scoreInfo.getChemistryGrade() && scoreInfo.getChemistryGrade() >= 0) {
            name.add(StudentClassEnum.CHEMISTRY_GRADE.getDesc());
        }

        if (null != scoreInfo.getBiologyGrade() && scoreInfo.getBiologyGrade() >= 0) {
            name.add(StudentClassEnum.BIOLOGY_GRADE.getDesc());
        }

        if (null != scoreInfo.getHistoryGrade() && scoreInfo.getHistoryGrade() >= 0) {
            name.add(StudentClassEnum.HISTORY_GRADE.getDesc());
        }

        if (null != scoreInfo.getGeographyGrade() && scoreInfo.getGeographyGrade() >= 0) {
            name.add(StudentClassEnum.GEOGRAPHY_GRADE.getDesc());
        }

        if (null != scoreInfo.getPoliticsGrade() && scoreInfo.getPoliticsGrade() >= 0) {
            name.add(StudentClassEnum.POLITICS_GRADE.getDesc());
        }

        return name;
    }

    /**
     * 获取专业名称
     *
     * @param schemeVo 专业参数
     * @return 专业名称列表
     */
    private List<String> getProfessionName(SchemeVo schemeVo) {
        //把字符专ids业转为数组
        List<Integer> professionIds = StringUtil.strToArray(schemeVo.getProfessionIds());
        return professionMapper.getNameByIds(professionIds);
    }


    /**
     * 补充不足数量的志愿数
     *
     * @param list   志愿ids
     * @param number 差的数量
     * @param max    最高分
     * @param min    最低分
     * @return 集合
     */
    private List<Volunteer> fillUpVolunteer(List<Volunteer> list, Integer number, double max, double min, Integer type, List<String> classNames) {

        String classStr = classNames.stream().map(String::valueOf).collect(Collectors.joining(","));

        //已存在的志愿ids
        List<Integer> volunteerIds = list.stream().map(Volunteer::getId).collect(Collectors.toList());

        //需要补充的志愿条数
        int totalNumber = number - list.size();

        Map<String, Object> map = new HashMap<>(16);
        map.put("volunteerIds", volunteerIds);
        map.put("upGrade", max);
        map.put("downGrade", min);
        map.put("type", type);
        map.put("classNames", classNames);

        List<Volunteer> fillList = volunteerMapper.getFillList(map);
        List<Volunteer> resultList = new ArrayList<>();
        List<Volunteer> otherList = new ArrayList<>();
        List<Volunteer> endList = new ArrayList<>();
        Map<Double, List<Volunteer>> mapObj = fillList.stream().collect(Collectors.groupingBy(Volunteer::getLowestMark
        ));

        for (Map.Entry<Double, List<Volunteer>> entry : mapObj.entrySet()) {
            List<Volunteer> volunteers = entry.getValue();
            if (volunteers.size() > 2) {
                for (int i = 0; i < 2; i++) {
                    Random mRandom = new Random();
                    int count = mRandom.nextInt(volunteers.size());
                    //判断限制专业
                    String courseDemand = volunteers.get(count).getCourseDemand();
                    if (courseDemand.contains("和")) {
                        String[] course = courseDemand.split("和");
                        for (String co : course) {
                            if (!classStr.contains(co)) {
                                volunteers.remove(count);
                                break;
                            }
                        }
                    } else {
                        resultList.add(volunteers.get(count));
                        volunteers.remove(count);
                    }
                }
                otherList.addAll(volunteers);
            } else {
                resultList.addAll(volunteers);
            }

            if (resultList.size() >= totalNumber) {
                Collections.shuffle(resultList);
                return resultList.subList(0, totalNumber);
            }
        }

        Map<Double, List<Volunteer>> mapObje = otherList.stream().collect(Collectors.groupingBy(Volunteer::getLowestMark
        ));

        for (Map.Entry<Double, List<Volunteer>> entry : mapObje.entrySet()) {
            List<Volunteer> volunteers = entry.getValue();
            if (volunteers.size() > 2) {
                for (int i = 0; i < 2; i++) {
                    Random mRandom = new Random();
                    int count = mRandom.nextInt(volunteers.size());
                    //判断限制专业
                    String courseDemand = volunteers.get(count).getCourseDemand();
                    if (courseDemand.contains("和")) {
                        String[] course = courseDemand.split("和");
                        for (String co : course) {
                            if (!classStr.contains(co)) {
                                volunteers.remove(count);
                                break;
                            }
                        }
                    } else {
                        resultList.add(volunteers.get(count));
                        volunteers.remove(count);

                    }
                }
                endList.addAll(volunteers);
            } else {
                resultList.addAll(volunteers);
            }

            if (resultList.size() >= totalNumber) {
                Collections.shuffle(resultList);
                return resultList.subList(0, totalNumber);
            }
        }

        //从剩下的数据中随机抽取，添满数组
        Collections.shuffle(endList);
        if (resultList.size() < totalNumber) {
            int num = totalNumber - resultList.size();
            List<Volunteer> volunteers = new ArrayList<>();
            if (endList.size() > 0) {
                for (int i = 0; i < num; i++) {
                    Random mRandom = new Random();
                    int count = mRandom.nextInt(endList.size());
                    if (resultList.contains(endList.get(count))) {
                        endList.remove(count);
                        continue;
                    }
                    //判断限制专业
                    String courseDemand = endList.get(count).getCourseDemand();
                    if (courseDemand.contains("和")) {
                        String[] course = courseDemand.split("和");
                        for (String co : course) {
                            if (!classStr.contains(co)) {
                                endList.remove(count);
                                break;
                            }
                        }
                        continue;
                    }
                    volunteers.add(endList.get(count));
                    endList.remove(count);
                }
            }
            resultList.addAll(volunteers);
        }
        return resultList;
    }

    /**
     * 分页处理方法
     *
     * @param schemeVo 参数
     */
    private void pageHandle(SchemeQueryVo schemeVo) {
        Integer pageNum = schemeVo.getPageNo();
        Integer pageSize = schemeVo.getPageSize();
        if (null == pageSize || pageSize == 0) {
            pageSize = 10;
        }
        if (null == pageNum || pageNum == 0) {
            pageNum = 1;
        }
        Integer endNum = pageSize;
        Integer startNum = (pageNum - CommonAttributes.NUM_ONE) * pageSize;
        schemeVo.setEndNum(endNum);
        schemeVo.setStartNum(startNum);
        schemeVo.setPageNo(pageNum);
        schemeVo.setPageSize(pageSize);
    }


    /**
     * 超过限定展示方案条数，要帅选
     *
     * @param list  所以方案数据
     * @param total 限定展示方案条数
     * @return 筛选结果
     */
    private List<Volunteer> getVolunteerList(List<Volunteer> list, Integer total) {
        List<Volunteer> result = new ArrayList<>();
        List<Volunteer> otherList = new ArrayList<>();

        //分组，每组数量大于2的，取2条，小于等于则全取
        Map<Double, List<Volunteer>> map = list.stream().collect(Collectors.groupingBy(Volunteer::getLowestMark));

        for (Map.Entry<Double, List<Volunteer>> entry : map.entrySet()) {
            List<Volunteer> volunteers = entry.getValue();
            if (volunteers.size() > 2) {
                for (int i = 0; i < 2; i++) {
                    Random mRandom = new Random();
                    int number = mRandom.nextInt(volunteers.size());
                    result.add(volunteers.get(number));
                    volunteers.remove(number);
                }
                otherList.addAll(volunteers);
            } else {
                result.addAll(volunteers);
            }

            if (result.size() >= total) {
                return result.subList(0, total);
            }
        }

        //分组，每组数量大于2的，取2条，小于等于则全取
        Map<Double, List<Volunteer>> mapOther = otherList.stream().collect(Collectors.groupingBy(Volunteer::getLowestMark
        ));
        List<Volunteer> endList = new ArrayList<>();
        for (Map.Entry<Double, List<Volunteer>> entry : mapOther.entrySet()) {
            List<Volunteer> volunteers = entry.getValue();
            if (volunteers.size() > 2) {
                for (int i = 0; i < 2; i++) {
                    Random mRandom = new Random();
                    int number = mRandom.nextInt(volunteers.size());
                    result.add(volunteers.get(number));
                    volunteers.remove(number);
                }
                endList.addAll(volunteers);
            } else {
                result.addAll(volunteers);
            }

            if (result.size() >= total) {
                return result.subList(0, total);
            }
        }


        //从剩下的数据中随机抽取，添满数组
        if (result.size() < total) {
            int number = total - result.size();
            List<Volunteer> volunteers = new ArrayList<>();
            for (int i = 0; i < number; i++) {
                Random mRandom = new Random();
                int count = mRandom.nextInt(endList.size());
                if (result.contains(endList.get(count))) {
                    endList.remove(count);
                    continue;
                }
                volunteers.add(endList.get(count));
                endList.remove(count);
            }
            result.addAll(volunteers);
        }
        return result;
    }

    /**
     * 设置查询时间
     *
     * @param list 志愿列表
     */
    private void SetQueryTime(List<Volunteer> list) {
        if (!CollectionUtils.isEmpty(list)) {
            for (Volunteer volunteer : list) {
                volunteer.setQueryTime(new Date());
            }
        }
    }



}
