package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.constant.CommonAttributes;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.*;
import cn.wisenergy.model.dto.UserCommitDto;
import cn.wisenergy.model.dto.UserDto;
import cn.wisenergy.model.dto.UserInfoDto;
import cn.wisenergy.model.dto.UserKnownDto;
import cn.wisenergy.model.enums.GradeType;
import cn.wisenergy.model.enums.SchemeTypeEnums;
import cn.wisenergy.model.enums.StudentType;
import cn.wisenergy.model.vo.UserInfoVo;
import cn.wisenergy.model.vo.UserQueryVo;
import cn.wisenergy.model.vo.UserShowVo;
import cn.wisenergy.model.vo.UserVipCommitVo;
import cn.wisenergy.service.app.StaffUserVipService;
import cn.wisenergy.service.app.UserService;
import cn.wisenergy.service.common.VolunteerManager;
import cn.wisenergy.service.util.WxUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ Description: 用户接口实现
 * @ Author     : 86187
 * @ Date       : 2021/1/6 16:11
 */
@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UsersMapper, User> implements UserService {

    @Autowired
    private UsersMapper usersMapper;

    @Autowired
    private ScoreInfoMapper scoreInfoMapper;

    @Autowired
    private LoginRecordMapper loginRecordMapper;

    @Autowired
    private UserLimitMapper userLimitMapper;

    @Autowired
    private PayRecordMapper payRecordMapper;

    @Autowired
    protected VolunteerManager volunteerManager;
    @Autowired
    private AdmissionRuleMapper admissionRuleMapper;
    @Autowired
    private UserKnownMapper knownMapper;


    @Value("${wxPay.Applets_ID}")
    private String appid;

    @Value("${wxPay.SECRT_KEY}")
    private String key;

    private static final int ERROR_CODE = 1;

    @Override
    public R<PageInfo<UserInfoDto>> getUserList(UserQueryVo queryVo) {
        log.info("volunteer_service[]UserServiceImpl[]getUserList[]input.param.queryVo:" + queryVo);

        //创建参数容器
        Map<String, Object> map = new HashMap<>(4);

        //将参数放入容器中
        if (null != queryVo.getPageNo() || null != queryVo.getPageSize()) {
            map.put("pageNo", (queryVo.getPageNo() - 1) * queryVo.getPageSize());
            map.put("pageSize", queryVo.getPageSize());
        }
        map.put("startTime", queryVo.getStartTime());
        map.put("endTime", queryVo.getEndTime());
        map.put("userName", queryVo.getUserName());
        map.put("phone", queryVo.getPhone());

        //查询用户数据
        List<User> list1 = usersMapper.getListQuery(map);

        List<UserInfoDto> list = new ArrayList<>();

        //将集合遍历拿出用户id查询成绩信息
        for (User user : list1) {
            List<PayRecord> byIds = payRecordMapper.getByIds(String.valueOf(user.getId()));
            UserInfoDto userInfoDto = new UserInfoDto();
            //计算充值卡充值次数
            int rechargeTimes = 0;
            //支付宝充值总金额
            double alipayMoney = 0.00;
            //微信充值总金额
            double weChatMoney = 0.00;
            for (PayRecord payRecord : byIds) {
                if (payRecord.getType() == 1) {
                    alipayMoney += null == payRecord.getMoney() ? 0 : payRecord.getMoney();
                }
                if (payRecord.getType() == 2) {
                    weChatMoney += null == payRecord.getMoney() ? 0 : payRecord.getMoney();
                }
                if (payRecord.getType() == 3) {
                    rechargeTimes += 1;
                }
            }
            userInfoDto.setWeChatMoney(String.format("%.2f", weChatMoney));
            userInfoDto.setAlipayMoney(String.format("%.2f", alipayMoney));
            userInfoDto.setRechargeTimes(rechargeTimes);
            userInfoDto.setQueryLimit(user.getUsableLimit());

            //将消息进行同步
            BeanUtils.copyProperties(user, userInfoDto);
            userInfoDto.setUserId(user.getId());
            userInfoDto.setRegisterTime(user.getCreateTime());
            list.add(userInfoDto);
        }
        PageInfo<UserInfoDto> info = new PageInfo<>();
        info.setPageSize(null == queryVo.getPageSize() ? 0 : queryVo.getPageSize());
        info.setPageNum(null == queryVo.getPageNo() ? 0 : queryVo.getPageNo());
        info.setTotal(usersMapper.getUserNumbers(map));
        info.setList(list);

        return R.ok(info);
    }


    @Override
    public R<UserInfoVo> commitUserInfo(UserCommitDto userInfo) {
        log.info("volunteer-service[]UserServiceImpl[]commitUserInfo[]input.param.userInfo:" + userInfo);
        if (null == userInfo) {
            return R.error("入参为空!");
        }
        userInfo.setUserName(userInfo.getUserName().replace(" ", ""));
        //1、根据用户id,获取用户信息
        User user = usersMapper.getById(userInfo.getUserId());
        if (null == user) {
            return R.error("用户信息不存在!");
        }

        //2、要提交的用户信息
        user.setSchool(userInfo.getSchool());
        user.setSex(userInfo.getSex());
        user.setExamType(userInfo.getStudentType());
        user.setUserName(userInfo.getUserName());
        user.setIsDelete(0);
        user.setIsVip(2);
        user.setSource(userInfo.getSource());

        //3、要提交的用户的成绩信息
        ScoreInfo scoreInfo = userInfo.getScoreInfo();

        //检查副科
        R<Boolean> checkScore = volunteerManager.checkScore(scoreInfo);
        if (null != checkScore && checkScore.getCode() == ERROR_CODE) {
            return R.error(checkScore.getMessage());
        }

        ScoreInfo flagStatus = scoreInfoMapper.getByUserId(user.getId());
        int flag;
        if (null == flagStatus) {
            //新增
            flag = 1;
        } else {
            //编辑
            flag = 2;
            scoreInfo.setId(flagStatus.getId());
        }

        //4、更新用户信息和保存用户成绩
        boolean bool = volunteerManager.updateAndSaveUserInfo(user, scoreInfo, flag);
        if (!bool) {
            return R.error("添加用户信息失败!");
        }

        //返回信息
        UserInfoVo userInfoVo = new UserInfoVo();
        userInfoVo.setUserId(userInfo.getUserId());
        userInfoVo.setUserName(userInfo.getUserName());
        userInfoVo.setPhone(user.getPhone());
        userInfoVo.setSex(userInfo.getSex());
        userInfoVo.setStudentType(userInfo.getStudentType());
        userInfoVo.setUserName(userInfo.getUserName());
        return R.ok(userInfoVo);
    }

    @Override
    public R<Boolean> edit(UserCommitDto userDto) {
        if (null == userDto || StringUtils.isBlank(userDto.getUserName())) {
            return R.error("入参为空!");
        }
        userDto.setUserName(userDto.getUserName().replace(" ", ""));
        //1、根据用户id,获取用户信息
        User user = usersMapper.getById(userDto.getUserId());
        if (null == user) {
            return R.error("用户信息不存在!");
        }

        if(!StringUtils.isBlank(userDto.getVipMobile()) && !getUserInfo(userDto.getVipMobile(),userDto.getUserId())){
            return R.error("该电话已存在");
        }

        //2、要提交的用户信息
        user.setSchool(userDto.getSchool());
        user.setSex(userDto.getSex());
        user.setExamType(userDto.getStudentType());
        user.setUserName(userDto.getUserName());
        user.setVipMobile(userDto.getVipMobile());

        //3、要提交的用户的成绩信息
        ScoreInfo scoreInfo = scoreInfoMapper.getByUserId(userDto.getUserId());

        //检查副科
        R<Boolean> checkScore = volunteerManager.checkScore(userDto.getScoreInfo());
        if (null != checkScore && checkScore.getCode() == ERROR_CODE) {
            return R.error(checkScore.getMessage());
        }

        scoreInfo.setUserId(userDto.getUserId());
        scoreInfo.setBiologyGrade(userDto.getScoreInfo().getBiologyGrade());
        scoreInfo.setChemistryGrade(userDto.getScoreInfo().getChemistryGrade());
        scoreInfo.setCultureGrade(userDto.getScoreInfo().getCultureGrade());
        scoreInfo.setEnglishGrade(userDto.getScoreInfo().getEnglishGrade());
        scoreInfo.setGeographyGrade(userDto.getScoreInfo().getGeographyGrade());
        scoreInfo.setHistoryGrade(userDto.getScoreInfo().getHistoryGrade());
        scoreInfo.setLanguageGrade(userDto.getScoreInfo().getLanguageGrade());
        if(StringUtils.isBlank(userDto.getScoreInfo().getMajorGrade()+"")){
            scoreInfo.setMajorGrade(0.0);
        }else{
            scoreInfo.setMajorGrade(userDto.getScoreInfo().getMajorGrade());
        }
        scoreInfo.setMathGrade(userDto.getScoreInfo().getMathGrade());
        scoreInfo.setPhysicsGrade(userDto.getScoreInfo().getPhysicsGrade());
        scoreInfo.setPoliticsGrade(userDto.getScoreInfo().getPoliticsGrade());


        //4、更新用户信息和保存用户成绩
        boolean bool = volunteerManager.updateAndUserInfo(user, scoreInfo);

        return R.ok(bool);
    }

    @Override
    public R<Boolean> delete(Integer userId) {
        return null;
    }

    @Override
    public R<UserShowVo> getById(Integer id) {
        log.info("volunteer-service[]UserServiceImpl[]getById[]input.param.id:" + id);
        if (null == id) {
            return R.error("入参为空!");
        }

        //根据id,获取用户信息
        User user = usersMapper.getById(id);
        if (null == user) {
            return R.error("用户信息不存在!");
        }

        UserShowVo userShowVo = new UserShowVo();
        userShowVo.setId(user.getId());
        userShowVo.setSchool(user.getSchool());
        userShowVo.setSex(user.getSex());
        userShowVo.setStudentType(user.getExamType());
        userShowVo.setUserName(user.getUserName());
        if (user.getVipMobile() != null) {
            userShowVo.setVipMobile(user.getVipMobile());
        }
        //获取成绩信息
        QueryWrapper<ScoreInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", user.getId());
        ScoreInfo scoreInfo = scoreInfoMapper.selectOne(queryWrapper);
        if (null != scoreInfo) {
            userShowVo.setScoreInfo(scoreInfo);
            //文化课考生
            if (StudentType.CULTURE_STUDENT.getCode().equals(user.getExamType())) {
                userShowVo.setComprehensiveScore(scoreInfo.getCultureGrade());
                AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_CULTURE.getCode());
                if (null != firstRule) {
                    double firstCulture =scoreInfo.getCultureGrade()-(firstRule.getCurrentYearCulture() - firstRule.getCultureMin()) ;
                    if (firstCulture >= firstRule.getCultureMin()) {
                        userShowVo.setGradeType(GradeType.UNDERGRADUATE.getDescription());
                    } else {
                        userShowVo.setGradeType(GradeType.SPECIALTY.getDescription());
                    }
                }
            }
            //美术考生
            if (StudentType.ART_STUDENT.getCode().equals(user.getExamType())) {
                //综合成绩=专业成绩*750/300*70%+文化成绩*30%
                double score=scoreInfo.getMajorGrade() * 750 / 300 * 0.7 + scoreInfo.getCultureGrade() * 0.3;
                userShowVo.setComprehensiveScore(new BigDecimal(score).setScale(2, RoundingMode.UP).doubleValue());
                AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_ARTS.getCode());
                if (null != firstRule) {
                    double culture =scoreInfo.getCultureGrade()-(firstRule.getCurrentYearCulture() - firstRule.getCultureMin()) ;
                    double major =scoreInfo.getMajorGrade() -(firstRule.getCurrentYearMajor() - firstRule.getProfessionMin()) ;
                    if (culture >= firstRule.getCultureMin() && major >= firstRule.getProfessionMin()) {
                        userShowVo.setGradeType(GradeType.UNDERGRADUATE.getDescription());
                    } else {
                        userShowVo.setGradeType(GradeType.SPECIALTY.getDescription());
                    }
                }
            }
            //文学编导考生
            if (StudentType.LITERATURE_STUDENT.getCode().equals(user.getExamType())) {
                //综合成绩=专业成绩*750/300*70%+文化成绩*30%
                Double score=scoreInfo.getMajorGrade() * 750 / 300 * 0.7 + scoreInfo.getCultureGrade() * 0.3;
                userShowVo.setComprehensiveScore(new BigDecimal(score).setScale(2, RoundingMode.UP).doubleValue());
                AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_LITERATURE.getCode());
                //本科类比分 = 本年录取分 -去年录取分 + 考生分数
                if (null != firstRule) {
                    double culture =scoreInfo.getCultureGrade()-(firstRule.getCurrentYearCulture() - firstRule.getCultureMin()) ;
                    double major =scoreInfo.getMajorGrade() -(firstRule.getCurrentYearMajor() - firstRule.getProfessionMin()) ;
                    if (culture >= firstRule.getCultureMin() && major >= firstRule.getProfessionMin()) {
                        userShowVo.setGradeType(GradeType.UNDERGRADUATE.getDescription());
                    } else {
                        userShowVo.setGradeType(GradeType.SPECIALTY.getDescription());
                    }
                }
            }
            //体育考生
            if (StudentType.SPORTS_STUDENT.getCode().equals(user.getExamType())) {
                //综合成绩=专业成绩*750/100*70%+文化成绩*30%
                AdmissionRule firstRule = admissionRuleMapper.getByType(SchemeTypeEnums.UNDERGRADUATE_SPORTS.getCode());
                double total = scoreInfo.getMajorGrade() * 750 / 100 * 0.7 + scoreInfo.getCultureGrade() * 0.3 ;
                userShowVo.setComprehensiveScore(new BigDecimal(total).setScale(2, RoundingMode.UP).doubleValue());
                if (null != firstRule) {
                    double value = firstRule.getCurrentYearCulture() - firstRule.getCultureMin();
                    double grad=total - value;
                    if (grad >= firstRule.getCultureMin()) {
                        userShowVo.setGradeType(GradeType.UNDERGRADUATE.getDescription());
                    } else {
                        userShowVo.setGradeType(GradeType.SPECIALTY.getDescription());
                    }
                }
            }
        }
        return R.ok(userShowVo);
    }

    @Override
    public User getByPhone(String phone) {
        return null;
    }

    @Override
    public R<Boolean> createExcel(UserQueryVo queryVo, HttpServletResponse response) {
        log.info("UserServiceImpl[].createExcel[].input.param:" + queryVo);

        queryVo.setPageSize(null);
        queryVo.setPageNo(null);
        //生成Excel
        try {
            //  设置内容格式 以及 编码方式
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");

            List<UserInfoDto> list = getUserList(queryVo).getData().getList();

            //  使用java8新特性的stream流去处理数据，把空的数据过滤掉
            List<UserDto> resultBo = list.stream().filter(Objects::nonNull)
                    .map(t -> UserDto.builder()
                            .registerTime(t.getRegisterTime())
                            .source(t.getSource() == 1 ? "PC" : t.getSource() == 2 ? "H5" : "小程序")
                            .userName(t.getUserName() == null ? null : t.getUserName())
                            .phone(t.getPhone() == null ? null : t.getPhone())
                            .lastLoginTime(t.getLastLoginTime() == null ? null : t.getLastLoginTime())
                            .school(t.getSchool() == null ? null : t.getSchool())
                            .examType(t.getExamType() == null ? null : t.getExamType() == 1 ? "文化课考生" : t.getExamType() == 2 ? "美术生" : t.getExamType() == 3 ? "体育生" : "文学编导考生")
                            .sex(t.getSex() == null ? null : t.getSex() == 0 ? "男" : "女")
                            .rechargeTimes(t.getRechargeTimes())
                            .weChatMoney(t.getWeChatMoney())
                            .alipayMoney(t.getAlipayMoney())
                            .lastLoginTime(t.getLastLoginTime())
                            .cultureGrade(t.getCultureGrade())
                            .majorGrade(t.getMajorGrade())
                            .queryLimit(t.getQueryLimit())
                            .build()).collect(Collectors.toList());

            //创建文件名称
            long lon = System.currentTimeMillis();

            response.setHeader("Content-disposition", "attachment;filename=" + lon + ".xlsx");

            //  sheet名称
            EasyExcel.write(response.getOutputStream(), UserDto.class).sheet(Long.toString(lon)).doWrite(resultBo);
            return R.ok(0, true);
        } catch (Exception e) {
            return R.ok(1, false);
        }
    }

    @Override
    public R<List<User>> test() {
        List<String> names = new ArrayList<>();
        names.add("黎楚川");
        names.add("陈成");
        names.add("雷青松");
        List<User> list = usersMapper.test(names);
        return R.ok(list);
    }

    /**
     * 分页处理方法
     *
     * @param orderQueryVo 参数
     */
    private void pageHandle(UserQueryVo orderQueryVo) {
        Integer pageNum = orderQueryVo.getPageNo();
        Integer pageSize = orderQueryVo.getPageSize();
        if (null == pageSize || pageSize == 0) {
            pageSize = 10;
        }
        if (null == pageNum || pageNum == 0) {
            pageNum = 1;
        }
        Integer endNum = pageSize;
        Integer startNum = (pageNum - CommonAttributes.NUM_ONE) * pageSize;
        orderQueryVo.setEndNum(endNum);
        orderQueryVo.setStartNum(startNum);
        orderQueryVo.setPageNo(pageNum);
        orderQueryVo.setPageSize(pageSize);
    }

    @Override
    public R<UserInfoVo> commitUserVipInfo(UserVipCommitVo userInfo) {
        log.info("volunteer-service[]UserServiceImpl[]commitUserVipInfo[]input.param.userInfo:" + userInfo);
        if (null == userInfo || null == userInfo.getStaffId() || null == userInfo.getSource()) {
            return R.error("入参为空!");
        }

        User user = new User();
        //2、要提交的用户信息
        user.setSchool(userInfo.getSchool());
        user.setSex(userInfo.getSex());
        user.setExamType(userInfo.getStudentType());
        user.setUserName(userInfo.getUserName());
        user.setIsDelete(0);
        user.setIsVip(1);
        user.setId(userInfo.getUserId());
        user.setSource(userInfo.getSource());

        //3、要提交的用户的成绩信息
        ScoreInfo scoreInfo = userInfo.getScoreInfo();

        //检查副科
        R<Boolean> checkScore = volunteerManager.checkScore(scoreInfo);
        if (null != checkScore && checkScore.getCode() == ERROR_CODE) {
            return R.error(checkScore.getMessage());
        }

        //设置flag为添加用户
        userInfo.setFlag(1);

        //4、更新用户信息和保存用户成绩
        boolean bool = volunteerManager.updateAndSaveUserVipInfo(user, scoreInfo, userInfo.getFlag(), userInfo.getStaffId());
        if (!bool) {
            return R.error("添加用户信息失败!");
        }


        //返回信息
        UserInfoVo userInfoVo = new UserInfoVo();
        userInfoVo.setUserId(userInfo.getUserId());
        userInfoVo.setUserName(userInfo.getUserName());
        userInfoVo.setPhone(user.getPhone());
        userInfoVo.setSex(userInfo.getSex());
        userInfoVo.setStudentType(userInfo.getStudentType());
        userInfoVo.setUserName(userInfo.getUserName());
        return R.ok(userInfoVo);
    }

    @Override
    public R<String> appletBinding(String code, Integer userId) {
        if (StringUtils.isBlank(code) || userId == null) {
            return R.error("入参为空!");
        }
        Map<String, Object> map = WxUtil.getInfoByCode(code, appid, key);
        log.info("code解析数据：" + map);
        if (null == map.get("openid")) {
            return R.error(map.get("errmsg").toString());
        }
        User user = new User();
        user.setOpenid(map.get("openid").toString());
        user.setId(userId);
        user.setUpdateTime(new Date());
        int update = baseMapper.updateById(user);
        if (update > 0) {
            return R.ok("授权成功");
        }
        return R.error("授权失败");
    }

    @Override
    public R<UserInfoVo> getByOpenid(String code) {
        if (StringUtils.isBlank(code)) {
            return R.error("入参为空!");
        }
        Map<String, Object> map = WxUtil.getInfoByCode(code, appid, key);
        log.info("code解析数据：" + map);

        if (null == map.get("openid")) {
            return R.error(map.get("errmsg").toString());
        }
        Map<String, Object> maps = new HashMap<>(16);
        maps.put("openid", map.get("openid").toString());
        User user = usersMapper.selectOne(maps);

        if (null == user) {
            return R.error("该用户还没有授权");
        }
        UserInfoVo userInfoVo = new UserInfoVo();
        userInfoVo.setUserId(user.getId());
        userInfoVo.setUserName(user.getUserName());
        userInfoVo.setPhone(user.getPhone());
        userInfoVo.setSex(user.getSex());
        userInfoVo.setStudentType(user.getExamType());
        return R.ok(userInfoVo);
    }

    @Override
    public R<String> quitApplet(Integer userId) {
        if (userId == null) {
            return R.error("入参为空!");
        }
        User user = new User();
        user.setId(userId);
        user.setOpenid("");
        user.setUpdateTime(new Date());
        int update = baseMapper.updateById(user);
        if (update > 0) {
            return R.ok("退出成功");
        }
        return R.error("退出失败");
    }

    @Override
    public R<UserKnownDto> getUserKnown() {
        QueryWrapper<UserKnown> userKnownQueryWrapper = new QueryWrapper<>();
        userKnownQueryWrapper.eq("is_delete",0);
        userKnownQueryWrapper.eq("is_show",1);
        userKnownQueryWrapper.orderByDesc("create_time");
        userKnownQueryWrapper.last("limit 0,1 ");
        UserKnown userKnown = knownMapper.selectOne(userKnownQueryWrapper);
        UserKnownDto userKnownDto = new UserKnownDto();
        userKnownDto.setContent(userKnown.getContent());
        userKnownDto.setId(userKnown.getId());
        userKnownDto.setCreateTime(userKnown.getCreateTime());
        return R.ok(userKnownDto);
    }

    public boolean getUserInfo(String vipMobile,Integer userId){
        Map<String, Object> map = new HashMap<>(16);
        map.put("vip_mobile",vipMobile);
        User user = usersMapper.selectOne(map);
        return null == user || user.getId().equals(userId);
    }

}

